#!/bin/sh

# To be included in SNMP config (etc/config/snmpd):
# config pass
#        option miboid   1.3.6.1.3.999.2.1
#        option prog     /usr/bin/getwlaninfo.sh
#        option persist  0
#
# SNMPv2-SMI::experimental.999.2.1.1.1 = STRING: "00:0b:6b:2f:a4:47"	# remote Station MAC
# SNMPv2-SMI::experimental.999.2.1.2.1 = INTEGER: -76			# RX Signal
# SNMPv2-SMI::experimental.999.2.1.3.1 = STRING: "54.0"			# TX Bitrate
# SNMPv2-SMI::experimental.999.2.1.4.1 = INTEGER: 28400			# Incative time
# SNMPv2-SMI::experimental.999.2.1.5.1 = Counter32: 39793967		# RX Bytes
# SNMPv2-SMI::experimental.999.2.1.6.1 = Counter32: 567465		# RX Packets
# SNMPv2-SMI::experimental.999.2.1.7.1 = Counter32: 784278242		# TX Bytes
# SNMPv2-SMI::experimental.999.2.1.8.1 = Counter32: 90357548		# TX Packets
# SNMPv2-SMI::experimental.999.2.1.9.1 = STRING: "wlan0"		# Interface name
#


PLACE=".1.3.6.1.3.999.2.1"	# last idx may be used for multiple instances (WLAN interfaces ;-)
#&&&LAN_IF="wlan0"
WLAN_IFS=$(ifconfig -a|grep wlan|grep -v mon|cut -f 1 -d' ')
WLAN_IFS="wlan0 wlan1"

# $PLACE.col.idx

CACHEFILE="/tmp/getwlaninfo.cache"
CACHETIMEOUT=20

REQ="$2"

# to allow testing on non-ALIX boards (no iw)
if ! which iw >/dev/null 2>&1; then
IWCMD_FAKE="echo -e
Station 00:0b:6b:2f:a2:c8 (on wlan0)\n
        inactive time:  1100 ms\n
        rx bytes:       783967238\n
        rx packets:     90353147\n
        tx bytes:       870546\n
        tx packets:     11799\n
        signal:         -45 dBm\n
        tx bitrate:     54.0 MBit/s\n
Station 00:0b:6b:2f:a2:ff (on wlan0)\n
        inactive time:  100 ms\n
        rx bytes:       783967238\n
        rx packets:     90353147\n
        tx bytes:       870546\n
        tx packets:     11799\n
        signal:         -85 dBm\n
        tx bitrate:     16.2 MBit/s\n
"
fi


###########################################################################
# Functions
###########################################################################
#
# Output the requested value
do_output () {
    if [ "$REC" == "" ]; then return; fi

    # prepare output:
    OID=$(echo $REC|sed -e 's/\(.*\)|.|.*/\1/')
    TYPE=$(echo $REC|sed -e 's/.*|\(.\)|.*/\1/')
    VALUE=$(echo $REC|sed -e 's/.*|.|\(.*\)/\1/')

    # <iod>
    echo $OID
    # <type>
    case "$TYPE" in
        S)	echo "string";;
        I)	echo "integer";;
        O)	echo "objectid";;
        T)	echo "timeticks";;
        A)	echo "ipaddress";;
        C)	echo "counter";;
        G)	echo "gauge";;
        *)	echo "string";;
    esac
    # <value>
    echo $VALUE

    return;
}

#
# Process a line
do_process_line () {
        if   echo $LINE|egrep -q '^Station '; then
            IDX=$(($IDX+1))
            C_STN=$(echo $LINE|sed -e 's/.* \(..:..:..:..:..:..\) .*/\1/')
            C_LNAME=$(echo $LINE|sed -e 's/.* \(..:..:..:..:..:..\).*(on \(.*\))/\2/')
        elif echo $LINE|egrep -q 'tx bitrate:'; then
            EOR=1
            C_RATE=$(echo $LINE|sed -e 's/.* \([0-9]\+\.[0-9]*\).*/\1/')
        elif echo $LINE|egrep -q 'signal:'; then
            C_SIG=$(echo $LINE|sed -e 's/.* \(\-\?[0-9]\+\.\?[0-9]*\).*/\1/')
        elif echo $LINE|egrep -q 'inactive time:'; then
            C_INACT=$(echo $LINE|sed -e 's/.* \(\-\?[0-9]\+\.\?[0-9]*\).*/\1/')
        elif echo $LINE|egrep -q 'rx bytes:'; then
            C_RXB=$(echo $LINE|sed -e 's/.* \(\-\?[0-9]\+\.\?[0-9]*\).*/\1/')
        elif echo $LINE|egrep -q 'rx packets:'; then
            C_RXP=$(echo $LINE|sed -e 's/.* \(\-\?[0-9]\+\.\?[0-9]*\).*/\1/')
        elif echo $LINE|egrep -q 'tx bytes:'; then
            C_TXB=$(echo $LINE|sed -e 's/.* \(\-\?[0-9]\+\.\?[0-9]*\).*/\1/')
        elif echo $LINE|egrep -q 'tx packets:'; then
            C_TXP=$(echo $LINE|sed -e 's/.* \(\-\?[0-9]\+\.\?[0-9]*\).*/\1/')
        fi

    return;
}

#
# Clean the row cache
do_clean_row_cache () {
    EOR=0
    # clean record
    C_STN=""
    C_SIG=""
    C_RATE=""
    C_INACT=""
    C_RXB=""
    C_RXP=""
    C_TXB=""
    C_TXP=""
    C_LNAME=""
    C_LMAC=""

    return;
}

#
# Prepare additional fields
do_prepare_record () {
    C_LMAC=$(ifconfig $C_LNAME| grep HWaddr | \
             sed -e 's/.*HWaddr \(..:..:..:..:..:..\)/\1/' -e 'y/ABCDEF/abcdef/')
    return;
}
#
# Print one row of the SNMP table
do_print_row () {
    _idx=$(printf "%4.4u" $IDX)	# make index 4 digit with leading '0'
    echo "$PLACE.01.$_idx|S|$C_STN"             # COl1: remote Station MAC (String)
    echo "$PLACE.02.$_idx|I|$C_SIG"             # COl2: RX Signal (String)
    echo "$PLACE.03.$_idx|S|$C_RATE"            # COl3: TX Bitrate (String)
    echo "$PLACE.04.$_idx|I|$C_INACT"           # COl4: Inactive Time
    echo "$PLACE.05.$_idx|C|$C_RXB"             # COl5: RX Bytes
    echo "$PLACE.06.$_idx|C|$C_RXP"             # COl6: RX Packets
    echo "$PLACE.07.$_idx|C|$C_TXB"             # COl7: TX Bytes
    echo "$PLACE.08.$_idx|C|$C_TXP"             # COl8: TX Packets
    echo "$PLACE.09.$_idx|S|$C_LNAME"           # COl9: local Interface name
    echo "$PLACE.10.$_idx|S|$C_LMAC"            # COl10: local MAC

    do_clean_row_cache
    unset _idx
    return;
}


###########################################################################
# Main Program
###########################################################################

# catch write attempts
if [ "$1" = "-s" ]; then
  echo "not-writable"
  exit 0
fi

# check if temp file has expired?
if [ -f $CACHEFILE ]; then
    C_TIME=$(date +%s -r $CACHEFILE)
    NOW=$(date +%s)
    C_AGE=$(($NOW - $C_TIME))
else
    C_AGE=9999999
fi

# if expired, re-create with curent data
if [ $C_AGE -gt $CACHETIMEOUT ]; then
    # clear & refresh cache file
    echo "$PLACE|" > $CACHEFILE

    IDX=0
    do_clean_row_cache

    for each in $WLAN_IFS; do
        WLAN_IF=$each


        IWCMD="iw $WLAN_IF station dump"
        if [ "$IWCMD_FAKE" != "" ]; then IWCMD=$IWCMD_FAKE; fi

        $IWCMD
    done |
    while read LINE; do
        # skip empty line
        if [ "$LINE" == "" ]; then continue; fi

        # evaluate line and fill row cache
        do_process_line

        if [ $EOR == 1 ]; then
            do_prepare_record
            # print record
            do_print_row
        fi
    # now this will need some explication.
    # As sort can not sort OIDs and going for one and two digit
    # column an row numbers they would be messed up with sort.
    # To avoid this, endagered values (columns are rows) are
    # expanded to be of a fixed number of digits with leading zeros,
    # this makes sort behave correctly.
    # After sorting those excessive leading '0' will be stripped
    # away with the SED magic below:
    done | sort | sed -e "s/\.0\+\([^|]\)/\.\1/g"  >> $CACHEFILE

    # add one emty line (required for grep -A below)
    echo >> $CACHEFILE
fi

# process request
REC=""
if [ "$1" = "-n" ]; then
   # getnext
   REC=$(grep -A 1 "$REQ|" $CACHEFILE|tail -1)
   # somehow I should catch the next entry even if I'm stuck "in between"
   # e.g. trying to dump the column of a table
   # snmpwalk -v2c -cpublic alix-1 .1.3.6.1.3.999.2.1.1
   if [ "$REC" == "" ]; then
      REQ="${REQ}.1"
      REC=$(grep "$REQ|" $CACHEFILE|tail -1)          
   fi
else
   # get
   REC=$(grep "$REQ|" $CACHEFILE)
fi

# write SNMP output
do_output

exit 0

###########################################################################
